summaryrefslogtreecommitdiff
path: root/actions/api.php
diff options
context:
space:
mode:
authorzach <zach@copley.name>2008-07-14 04:07:41 -0400
committerzach <zach@copley.name>2008-07-14 04:07:41 -0400
commitcc55a45f4bad24d5921b4575389650e93efe6605 (patch)
tree6a3933cf4812dfc08938f71244eb382065c5d6da /actions/api.php
parent1eae7e0ae119aaf4073dd09d6233551f7714a7cd (diff)
Second shot at figuring out action dispatch for Twitter-compatible API
darcs-hash:20080714080741-ca946-1b2f8c026f433e27a30abc104ca2dba11b9b662e.gz
Diffstat (limited to 'actions/api.php')
-rw-r--r--actions/api.php85
1 files changed, 52 insertions, 33 deletions
diff --git a/actions/api.php b/actions/api.php
index 21404e331..2e7ed3558 100644
--- a/actions/api.php
+++ b/actions/api.php
@@ -19,18 +19,37 @@
if (!defined('LACONICA')) { exit(1); }
-// XXX: Not sure of terminology yet... maybe call things "api_methods" insteads of "commands"
-
class ApiAction extends Action {
+ var $nickname;
+ var $content_type;
+ var $api_arg;
+ var $api_method;
+ var $api_action;
+
function handle($args) {
parent::handle($args);
- $command = $this->arg('command');
+ $this->api_action = $this->arg('apiaction');
+ $method = $this->arg('method');
+ $argument = $this->arg('argument');
- # XXX Maybe check to see if the command actually exists first
+ if (isset($argument)) {
+ $cmdext = explode('.', $argument);
+ $this->api_arg = $cmdext[0];
+ $this->api_method = $method;
+ $this->content_type = $cmdext[1];
+ } else {
+ #content type will be an extension on the method
+ $cmdext = explode('.', $method);
+ $this->api_method = $cmdext[0];
+ $this->content_type = $cmdext[1];
+ }
- if($this->requires_auth($command)) {
+ # common_debug("apiaction = $this->api_action, method = $this->api_method, argument = $this->api_arg, ctype = $this->content_type");
+
+ # XXX Maybe check to see if the command actually exists first?
+ if($this->requires_auth()) {
if (!isset($_SERVER['PHP_AUTH_USER'])) {
# This header makes basic auth go
@@ -44,49 +63,49 @@ class ApiAction extends Action {
$user = common_check_user($nickname, $password);
if ($user) {
- $this->process_command($command, $nickname, $password);
+ $this->nickname = $nickname;
+ $this->process_command();
} else {
# basic authentication failed
common_show_basic_auth_error();
}
}
-
} else {
- $this->process_command($command);
- }
+ $this->process_command();
+ }
}
- # this is where we can dispatch off to api Class files
- function process_command($command, $nickname=NULL, $password=NULL) {
-
- $parts = explode('.', $command);
- $api_action = "api_$parts[0]";
- $extension = $parts[1]; # requested content type
-
- $api_actionfile = INSTALLDIR."/actions/$api_action.php";
-
- if (file_exists($api_actionfile)) {
- require_once($api_actionfile);
- $action_class = ucfirst($api_action)."Action";
+ function process_command() {
+ $action = "api$this->api_action";
+ $actionfile = INSTALLDIR."/actions/$action.php";
+ if (file_exists($actionfile)) {
+ require_once($actionfile);
+ $action_class = ucfirst($action)."Action";
$action_obj = new $action_class();
- # need to pass off nick and password and stuff ... put in $args? constructor?
- # pull from $_REQUEST later?
- call_user_func(array($action_obj, 'handle'), $_REQUEST);
- } else {
-
- # need appropriate API error functs
- print "\nerror!\n";
+ if (method_exists($action_obj, $this->api_method)) {
+
+ $apidata = array( 'content-type' => $this->content_type,
+ 'api_method' => $this->api_method,
+ 'api_arg' => $this->api_arg,
+ 'nickanme' => $htis->nickanme);
+
+ call_user_func(array($action_obj, $this->api_method), $_REQUEST, $apidata);
+ # all API methods should exit()
+ }
}
+ common_user_error("API method not found!", $code=404);
}
+
# Whitelist of API methods that don't need authentication
- function requires_auth($command) {
-
- # The only command that doesn't in Twitter's API is public_timeline
- if (ereg('^public_timeline.*$', $command)) {
+ function requires_auth() {
+ static $noauth = array( 'statuses/public_timeline',
+ 'help/test',
+ 'help/downtime_schedule');
+ if (in_array("$this->api_action/$this->api_method", $noauth)) {
return false;
- }
+ }
return true;
}